@@ -266,6 +266,30 @@ get_token (GDBusMethodInvocation *invocation)
                      interface, method, G_STRLOC);
         }
     }
+  else if (strcmp (interface, "org.freedesktop.portal.Location") == 0)
+    {
+      if (strcmp (method, "CreateSession") == 0 )
+        {
+          options = g_variant_get_child_value (parameters, 0);
+        }
+      else if (strcmp (method, "SelectDetails") == 0)
+        {
+          options = g_variant_get_child_value (parameters, 1);
+        }
+      else if (strcmp (method, "Start") == 0)
+        {
+          options = g_variant_get_child_value (parameters, 2);
+        }
+      else
+        {
+          g_warning ("Support for %s::%s missing in %s",
+                     interface, method, G_STRLOC);
+        }
+    }
+  else if (strcmp (interface, "org.freedesktop.portal.Settings") == 0)
+    {
+      // no request objects
+    }
   else
     {
       g_print ("Support for %s missing in " G_STRLOC, interface);
