@@ -335,9 +335,27 @@ select_sources_done (GObject *source_object,
     }
 }
 
+static gboolean
+validate_device_types (const char *key,
+                       GVariant *value,
+                       GVariant *options,
+                       GError **error)
+{
+  guint32 types = g_variant_get_uint32 (value);
+
+  if ((types & ~(1 | 2)) != 0)
+    {
+      g_set_error (error, XDG_DESKTOP_PORTAL_ERROR, XDG_DESKTOP_PORTAL_ERROR_INVALID_ARGUMENT,
+                   "Unsupported device type: %x", types & ~(1 | 2));
+      return FALSE;
+    } 
+
+  return TRUE;
+}
+
 static XdpOptionKey screen_cast_select_sources_options[] = {
-  { "types", G_VARIANT_TYPE_UINT32 },
-  { "multiple", G_VARIANT_TYPE_BOOLEAN },
+  { "types", G_VARIANT_TYPE_UINT32, validate_device_types },
+  { "multiple", G_VARIANT_TYPE_BOOLEAN, NULL },
 };
 
 static gboolean
@@ -435,9 +453,14 @@ handle_select_sources (XdpScreenCast *object,
   request_export (request, g_dbus_method_invocation_get_connection (invocation));
 
   g_variant_builder_init (&options_builder, G_VARIANT_TYPE_VARDICT);
-  xdp_filter_options (arg_options, &options_builder,
-                      screen_cast_select_sources_options,
-                      G_N_ELEMENTS (screen_cast_select_sources_options));
+  if (!xdp_filter_options (arg_options, &options_builder,
+                           screen_cast_select_sources_options,
+                           G_N_ELEMENTS (screen_cast_select_sources_options),
+                           &error))
+    {
+      g_dbus_method_invocation_return_gerror (invocation, error);
+      return TRUE;
+    }
 
   g_object_set_qdata_full (G_OBJECT (request),
                            quark_request_session,
@@ -591,9 +614,9 @@ static const struct pw_remote_events remote_events = {
 void
 pipewire_remote_destroy (PipeWireRemote *remote)
 {
-  g_clear_pointer (&remote->remote, (GDestroyNotify)pw_remote_destroy);
-  g_clear_pointer (&remote->core, (GDestroyNotify)pw_core_destroy);
-  g_clear_pointer (&remote->loop, (GDestroyNotify)pw_main_loop_destroy);
+  g_clear_pointer (&remote->remote, pw_remote_destroy);
+  g_clear_pointer (&remote->core, pw_core_destroy);
+  g_clear_pointer (&remote->loop, pw_main_loop_destroy);
   g_clear_error (&remote->error);
 
   g_free (remote);
@@ -857,7 +880,7 @@ start_done (GObject *source_object,
             {
               g_warning ("Failed to process results: %s", error->message);
               g_clear_error (&error);
-              g_clear_pointer (&results, (GDestroyNotify)g_variant_unref);
+              g_clear_pointer (&results, g_variant_unref);
               response = 2;
               should_close_session = TRUE;
             }
